c  **** ATTENTION : THIS PROGRAM USES FORTRAN NAME UP TO 10 CHARACTERS LONG ***
C
c
c
c
c
c
c
c
c                       o..............o
c                       :              :
c                   a(i-1,j)           :
c                   u(i,j)             :
c                *------>------*   S11, S22(i,j), pression, divergence
c                |      :      |   u**2, v**2, cc(i,j)
c                |      :      |       :
c                |     S12(i,j)| v(i,j):
c                ^      o..... ^.......o
c                |             | c(i,j-1) (c IS NOT cc) [a, c \simeq 1/\rho]
c                |             |
c                |             |
c                *------>------*
c   
c  
c      SKETCH OF TOP-BOTTOM BOUNDARY CONDITIONS
c   
c       *-----*------*------*------*  j =  ny   p(i,ny)
c       |     |      |      |      |
c   ....^.....^......^......^......^....        v(i,ny) = 0
c       |     |      |      |      |
c       |---------------------->---
c       |     |      |      |      |  j = ny-1
c       |     |      |      |      ^
c       |--------------------------|
c       |     |      |      |      |
c   
c   
c       |     |      |      |      |
c       |--------------------------
c       |     |      |      |      |
c       |     |      |      |  u(n-1,2)
c       |---------------------->-- |  p(n-1,2)            | Boundary conditions
c       |     |      |      |      |                      |    u(i,1) = u(i,2)
c   ....^..........................^...  y = 0 v(i,2)     |    p(i,1) = p(i,2) 
c       |     |      |      |      |                      |    v(i,2) = 0
c       *-->--*------*------*-->-- *   j=1                   S12(i,2) = 0 
c       |     |      |      |      |   p(n-1,1)              S12(i,ny) = 0
c       ^     ^      ^      ^      ^   v(n-1,1)
c       v(2,1)
c   
c   Sites on or outside the dotted lines are used to force boundary conditions
c   There is also one dummy column on each side (not represented).
c
c
c   u,v(i,j)  the physical velocities *tau/h
c
c
      SUBROUTINE momentum(rhou,rhov,u,v,cc,p,a,c,ei,
     *                    S11,S22,S12,w1,w2,tau,mugas,muliq,
     *                    rhogas,g,nx,ny,outdiv,t,parint,pardpr,COMM2D)
      include 'undefined.h'
      include 'mpif.h'
      INTEGER nx,ny,outdiv,t,ifetch,nres
      INTEGER PARINT(100),PPRANK,NEIGHB(8),COMM2D,IERR00
      DOUBLE PRECISION PARDPR(100),PCOORS(4),OUTVAL
      DOUBLE PRECISION tau, h, mugas, muliq, rhogas
      DOUBLE PRECISION rhou(nx,ny), rhov(nx,ny)
      DOUBLE PRECISION u(nx,ny), v(nx,ny), cc(nx,ny), p(nx,ny)
      DOUBLE PRECISION S11(nx,ny),S12(nx,ny),S22(nx,ny)
      DOUBLE PRECISION w1(nx,ny),w2(nx,ny)
      DOUBLE PRECISION a(nx,ny), c(nx,ny), ei(nx,ny)
      INTEGER i,j,imax,jmax,cycles,ncycle,npre,npost,horgrav,relax_times
     * ,printres,relax_times_old,ncycle_old,CAPDENS
      DOUBLE PRECISION mucst1ctr, mucst2ctr, mucst1, mucst2
      DOUBLE PRECISION rhocst1, rhocst1ov2
      DOUBLE PRECISION hi2, sumfield, fmodmax,normmean
      DOUBLE PRECISION tauh2,g,tau2h
      DOUBLE PRECISION s1,s2,t1,t2
      DOUBLE PRECISION findninf,div_total
      DOUBLE PRECISION epsilon,mineps,maxeps,xstdgrav,ystdgrav,norm_old
      COMMON /cycling/ mineps,maxeps,cycles,ncycle,npre,npost,nres,
     *relax_times,norm_old,printres,relax_times_old,ncycle_old

c
      horgrav=0
C.... Lines inserted by Kike for MPI version
      PPRANK   = PARINT(55)
      DO I = 1,8
         NEIGHB(I) = PARINT(60+I)
      ENDDO
      PCOORS(1) = PARDPR(54)
      PCOORS(2) = PARDPR(54) + PARDPR(52)
      PCOORS(3) = PARDPR(55)
      PCOORS(4) = PARDPR(55) + PARDPR(53)
C.... End of insersion
c
c  Numerical constants
c
      h=1.d0/(nx-2)
      tauh2 = tau/(h*h)
      tau2h = tau**2/h
      hi2 = 1.d0/(h*h)
c
c Constants used to calculated local mu
c Mu constant 1, centered
c
      mucst1ctr = tauh2*mugas
c
c Mu constant 2, centered
c
      mucst2ctr = 0.25d0*tauh2*( muliq - mugas)
c
c Mu constants 1 and 2, not centered
c
      mucst1 = 2.d0*tauh2*mugas
      mucst2 = 2.d0*tauh2*(muliq - mugas)
c
c Constants used to calculate local rho
c
      rhocst1 = 1.d0 - rhogas
      rhocst1ov2 =  .5d0*( 1.d0 - rhogas)
c
c  Gravity constants
c
      xstdgrav =  g*tau2h*horgrav
      ystdgrav =  g*tau2h*(1-horgrav) 
C
      do j=2,ny
         do i=2,nx
            S12(i,j) = - S12(i,j)
     *      + (mucst2ctr*(cc(i,j)+cc(i-1,j)+cc(i,j-1)+cc(i-1,j-1))
     *      +  mucst1ctr)
     *      * (u(i,j) - u(i,j-1) + v(i,j) - v(i-1,j))
         enddo
      enddo
      do j=2,ny-1
         do i=2,nx-1
            S11(i,j) = - S11(i,j)
     *      + (mucst2*cc(i,j) + mucst1)*(u(i+1,j) - u(i,j))
            S22(i,j) = - S22(i,j)
     *      + (mucst2*cc(i,j) + mucst1)*(v(i,j+1) - v(i,j))
         enddo
      enddo
C
C.... Next BC is substituted by one call to MAKEBCS
C.... and three calls to BUPDAT
C      call makebcsf(S11,S22,S12,nx,ny)
C
      CALL MAKEBCS(S12,PARINT,PARDPR,NX,NY)
      CALL BUPDAT(S11,PARINT,PARDPR,NX,NY,COMM2D)
      CALL BUPDAT(S12,PARINT,PARDPR,NX,NY,COMM2D)
      CALL BUPDAT(S22,PARINT,PARDPR,NX,NY,COMM2D)
C.... End of substitution.
C
C.... Following line inserted by Kike for MPI version
      CAPDENS   = PARINT(8)
      call afromc(a,c,ei,cc,rhocst1ov2,rhogas,capdens,nx,ny)
      CALL BUPDAT(a,PARINT,PARDPR,NX,NY,COMM2D)
      CALL BUPDAT(c,PARINT,PARDPR,NX,NY,COMM2D)
      CALL BUPDAT(ei,PARINT,PARDPR,NX,NY,COMM2D)
C
      do j=2,ny-1
         do i=2,nx-1
            w1(i,j) = u(i,j) + a(i-1,j)*(S12(i,j+1) - S12(i,j)
     *           + S11(i,j) - S11(i-1,j))
c     *          + xstdgrav
c
            w2(i,j) = v(i,j) + c(i,j-1)*(S12(i+1,j) -S12(i,j)
     *            + S22(i,j) - S22( i,j-1))
     *            + ystdgrav
         enddo
      enddo
C
C.... Next BC is substituted by one call to MAKEBCV
C.... and two calls to BUPDAT
C      call makebcf(w1,w2,nx,ny,t,c)
      CALL MAKEBCV(W1,W2,PARINT,PARDPR,NX,NY,COMM2D,T)
      CALL BUPDAT(W1,PARINT,PARDPR,NX,NY,COMM2D)
      CALL BUPDAT(W2,PARINT,PARDPR,NX,NY,COMM2D)
C.... End of substitution.
C
C  Jie Li's upwind version
C
      do j=2,ny-1
         do i=2,nx-1 
            t1=.5d0*(w2(i,j) + w2(i-1,j))
            t2=.5d0*(w2(i,j+1) + w2(i-1,j+1))
            s1=.5d0*(w1(i-1,j)+w1(i,j))
            s2=.5d0*(w1(i,j)+w1(i+1,j))
            u(i,j) =w1(i,j)
     *        +dmax1(t1,0.0d0)*w1(i,j-1)+dmin1(t1,0.0d0)*w1(i,j)
     *        -dmax1(t2,0.0d0)*w1(i,j)-dmin1(t2,0.0d0)*w1(i,j+1) 
     *        +dmax1(s1,0.0d0)*w1(i-1,j)+dmin1(s1,0.0d0)*w1(i,j)
     *        -dmax1(s2,0.0d0)*w1(i,j)-dmin1(s2,0.0d0)*w1(i+1,j)

            t1=0.5d0*(w1(i,j) + w1(i,j-1))
            t2=0.5d0*(w1(i+1,j) + w1(i+1,j-1))
            s1=0.5d0*(w2(i,j-1)+w2(i,j))
            s2=0.5d0*(w2(i,j)+w2(i,j+1))
            v(i,j) =w2(i,j)
     *        +dmax1(t1,0.0d0)*w2(i-1,j)+dmin1(t1,0.0d0)*w2(i,j)
     *        -dmax1(t2,0.0d0)*w2(i,j)-dmin1(t2,0.0d0)*w2(i+1,j)
     *        +dmax1(s1,0.0d0)*w2(i,j-1)+dmin1(s1,0.0d0)*w2(i,j)
     *        -dmax1(s2,0.0d0)*w2(i,j)-dmin1(s2,0.0d0)*w2(i,j+1)
         enddo
      enddo
c$$$      do j=2,ny-1
c$$$         do i=2,nx-1
c$$$            u(i,j) = w1(i,j)+ 0.25d0*(
c$$$     *      (w1(i,j) + w1(i,j-1))*(w2(i,j) + w2(i-1,j))
c$$$     *      -  (w1(i,j+1) + w1(i,j))*(w2(i,j+1) + w2(i-1,j+1))
c$$$     *      + (w1(i-1,j)+w1(i,j))**2 - (w1(i,j)+w1(i+1,j))**2)
c$$$            v(i,j) = w2(i,j) + 0.25d0*(
c$$$     *      (w1(i,j) + w1(i,j-1))*(w2(i,j) + w2(i-1,j))
c$$$     *      -  (w1(i+1,j) + w1(i+1,j-1))*(w2(i+1,j) + w2(i,j))
c$$$     *      + (w2(i,j-1)+w2(i,j))**2 - (w2(i,j)+w2(i,j+1))**2)        
c$$$         enddo
c$$$      enddo
c$$$
C
C.... Next BC is substituted by one call to MAKEBCV
C.... and two calls to BUPDAT
C      call makebcf(u,v,nx,ny,t,c)
      CALL MAKEBCV(U,V,PARINT,PARDPR,NX,NY,COMM2D,T)
      CALL BUPDAT(U,PARINT,PARDPR,NX,NY,COMM2D)
      CALL BUPDAT(V,PARINT,PARDPR,NX,NY,COMM2D)
C.... End of substitution.
c
c Now   w2=divergence
c       w1=pressure
c
      do j=2,ny-1
        do i=2,nx-1
          w2(i,j) = ( u(i+1,j) - u(i,j) + v(i,j+1) - v(i,j))*hi2
        enddo
      enddo
C
      if (t .le. 1) then
        write (6,*)'PPRANK= ',PPRANK,' t, divergence sum = ',t,
     &	             sumfield(w2,nx,ny)
        CALL MPI_ALLREDUCE(sumfield(w2,nx,ny),div_total,1,
     &                     MPI_DOUBLE_PRECISION,MPI_SUM,COMM2D,IERR00)
        write (6,*)'PPRANK= ',PPRANK,' t, divergence sum total = ',t,
     &	             div_total
      endif
C
      if(mod(t,outdiv).eq.0) then 
        write(6,*)'PPRANK= ',PPRANK,' t, ncycle max = ,div before = ',
     &            t,ncycle,normmean(w2,nx,ny)
      endif
C
      call mglin(rhocst1ov2,rhogas,nx,ny,parint,pardpr,COMM2D)
C
C
      do j=2,ny-1
         do i=2,nx-1
            u(i,j) = u(i,j) -  (p(i,j) - p(i-1,j))*a(i-1,j)
            v(i,j) = v(i,j) -  (p(i,j) - p(i,j-1))*c(i,j-1)
         enddo
      enddo
C.... Next BC is substituted by one call to MAKEBCV
C.... and two calls to BUPDAT
C      call makebcf(u,v,nx,ny,t,c)
      CALL MAKEBCV(U,V,PARINT,PARDPR,NX,NY,COMM2D,T)
      CALL BUPDAT(U,PARINT,PARDPR,NX,NY,COMM2D)
      CALL BUPDAT(V,PARINT,PARDPR,NX,NY,COMM2D)
C.... End of substitution.
C
      if(mod(t,outdiv).eq.0) then
        do j=2,ny-1
          do i=2,nx-1
            w2(i,j) = (u(i+1,j)-u(i,j)+v(i,j+1)-v(i,j))*hi2
          enddo
        enddo
        write(6,*)'PPRANK= ',PPRANK,' t, ncycles made=, div=',
     &	           t,cycles,normmean(w2,nx,ny)
      endif
      return
C
 100  format(g10.7)
      end
