      SUBROUTINE readglob(parint,pardpr)
C
C     Name: read globals
C     Author: Stphane Zaleski
C     Objective: New initialisation subroutine
C     Called by: Spatial
C     It calls: none
C     Modifications : Anthony Leboissetier (09/01) for the MPI version
C
      include 'undefined.h'
      INTEGER tmax,tprint,i
      DOUBLE PRECISION t1,t2,threshold,vel1,vel2, tau1
      INCLUDE 'size.h'
      INCLUDE 'global.f'
      DOUBLE PRECISION lambdasi,tausi,sigmasi,rholiqsi,rhogassi,
     *  viscgassi, viscliqsi, timescalesi, xvelsi, yvelsi,
     *     rholiq, gSI,duSI,dutrSI,circulSI,circul, spmul, spmug
      INTEGER parint(100),restart,cycles,nres,printres,relax_times,
     &        capdens,horgrav,printdiv,tvelo,cinalg,nfilter,tsave
      DOUBLE PRECISION pardpr(100),r1,d,r2,mineps
c
c circul passed directly to inivort
c
      common /COMVORT/ circul
c
c parameters passed to in_and_out
c
      common /INOUT/ vel1,vel2, tau1
c
c  First read SI values
c
c 1) compulsory parameters
c
      if(ifetch("n1",nx).ne.1) then
         if(ifetch("nx",nx).ne.1) stop 'missing n1'
      endif
      if(ifetch("n2",ny).ne.1) then
         if(ifetch("ny",ny).ne.1) stop 'missing n2'
      endif
      if(dfetch("tau-si",tauSI).ne.1)   stop 'missing tau-si'
      if(dfetch("lambda-si",lambdaSI).ne.1)  then
        if(dfetch("L-si",lambdaSI).ne.1) 
     *      stop 'missing lambda-si or L-si'
      else
         if(dfetch("L-si",lambdaSI).eq.1) 
     *      stop 'Both L-si and lambda-si defined'
      endif
c
      if(dfetch("rhogas-si",rhogasSI).ne.1) stop 'missing rhogas-si'
      if(dfetch("rholiq-si",rholiqSI).ne.1) stop 'missing rholiq-si'
c
      if(ifetch("tmax",tmax).ne.1) stop 'missing tmax'
      if(ifetch("tprint",tprint).ne.1)   stop 'missing tprint'
c
c Optional parameters -- default values. 
c
      xvelSI=0.d0
      yvelSI=0.d0
      sigmaSI=0.d0
      i=dfetch("xvel-si",xvelSI)
      i=dfetch("yvel-si",yvelSI)
      i=dfetch("sigma-si",sigmaSI)
      if(dfetch("viscgas-si",viscgasSI).ne.1)  
     *  stop 'missing viscgas-si'
      if(dfetch("viscliq-si",viscliqSI).ne.1)  
     *  stop 'missing viscliq-si'
c
c    Now we have a scale of mass ( rholiqSI*L^3 ) and  of length (l-SI)
c
c  We determine the time scale from 
c   A) the velocity scale if present 
c   B) Otherwise, from the gravity scale gSI if present (I.E. we are not
c        in outer space)
c   C) If none of the above, timescale=1second.
c
c
      gSI=0.
      duSI=0.
      circulSI=0.
c
      i=dfetch("gravity-si",gSI)
      i=dfetch("deltau-si",duSI)
      i=dfetch("circul-si",circulSI)
c
c  Determine the timescale from duSI, if duSI is zero try 
c  other methods. 
c
      if(duSI.ne.0.d0) then
         timescaleSI = lambdaSI/duSI
         du = 1.d0
         g=gSI / (lambdaSI/(timescaleSI**2))
      else if(gSI.ne.0.d0) then
         timescaleSI= dsqrt(lambdaSI/dabs(gSI))
         g=1.d0*gSI/dabs(gSI)
         du=0.d0
      else 
         timescaleSI=1
         g=0.d0
         du=0.d0
      endif
C
C    We now convert everything into dimensionless code units 
C
      tau = tauSI/timescaleSI
      rholiq=1
      rhogas=rhogasSI/rholiqSI
      mugas=viscgasSI*timescaleSI/(rholiqSI*lambdaSI**2)
      muliq=viscliqSI*timescaleSI/(rholiqSI*lambdaSI**2)
      sigma=sigmaSI*timescaleSI**2/(rholiqSI*lambdaSI**3)
      xvel=xvelSI*timescaleSI/lambdaSI
      yvel=yvelSI*timescaleSI/lambdaSI
      circul = circulSI/(lambdaSI**2/timescaleSI)
c
c Compute values for common INOUT
c
      vel1=xvel
      vel2=xvel+du
      tau1=tau      
C      write(6,*) "deltau-si",duSI
C      write(6,*) "lambda-si",lambdaSI
C      write(6,*) "tau-si=",tausi
C      write(6,*) "sigma-si=",sigmasi
C      write(6,*) "rhogas-si=",rhogassi
C      write(6,*) "rholiqsi=",rholiqsi
C      write(6,*) "viscgas-si=",viscgassi
C      write(6,*) "viscliq-si=",viscliqsi
C      write(6,*) "timescale-si=",timescaleSI
C      write(6,*) "inner nozzle velocity",vel1
C      write(6,*) "outer nozzle velocity",vel2
         restart=0
         i=ifetch("restart",restart)
         if(ifetch("MG_cycles_bound",cycles).ne.1) 
     *      stop 'missing MG_cycles_bound'
         if(ifetch("nres",nres).ne.1) nres=0 
         if (ifetch("printres",printres).ne.1) printres=0
         i = dfetch("div_tolerance",mineps)
         if(ifetch("relaxfine",relax_times).ne.1) relax_times=4
         capdens=0
         i=ifetch("capdens",capdens)
         horgrav=0
         i = ifetch("horgrav",horgrav)
         printdiv=1
         i=ifetch("printdiv",printdiv)
	 i=ifetch("tvelo",tvelo)
	 i=ifetch("tsave",tsave)
         cinalg=3
         i = ifetch("cinalg",cinalg)
	 nfilter=0
         i=ifetch("nfilter",nfilter) 
C
C... Let's fill the PARINT and PARDPR arrays
C
         parint(1) = nx
         parint(2) = ny
         parint(3) = restart
         parint(4) = cycles
         parint(5) = nres
         parint(6) = printres
         parint(7) = relax_times
         parint(8) = capdens
         parint(9) = horgrav
         parint(10) = printdiv
         parint(11) = tvelo
         parint(12) = cinalg
         parint(13) = nfilter
	 parint(14) = tmax
	 parint(15) = tprint
	 parint(16) = tsave
C
         i = dfetch("inner_nozzle_radius",r1)
         i = dfetch("separator_width",d)
         i = dfetch("outer_nozzle_width",r2)
	 i = dfetch("div_tolerance",mineps)
C
         pardpr(1) = tau
         pardpr(2) = sigma
         pardpr(3) = rhogas
         pardpr(4) = muliq
         pardpr(5) = mugas
         pardpr(6) = r1
         pardpr(7) = d
         pardpr(8) = r2
         pardpr(9) = mineps
         pardpr(10) = vel1
         pardpr(11) = vel2
C
      return
      end
C
      SUBROUTINE printhist(tprint)
      include 'undefined.h'
      DOUBLE PRECISION vmax,cfl,dis,h,tauh
      INTEGER tprint
      INCLUDE 'size.h'
      INCLUDE 'global.f'
C
      h = 1.d0/(nx-2)
      tauh=tau/h
      vmax = dmax1(dabs(xvel),dabs(yvel))
      cfl = tau*vmax/h
      if(cfl.gt.MAXCFL) write(6,*) "cfl=",cfl
      write(6,*) "d3=", tprint*tau
      write(6,*) "tau=",tau
      write(6,*) "sigma=",sigma
      write(6,*) "rhogas=",rhogas
      write(6,*) "viscgas=",mugas
      write(6,*) "viscliq=",muliq
      write(6,*) "xvel=",xvel
      write(6,*) "yvel=",yvel
      write(6,*) "g=",g
C
      return
      end
